 /*
  * Khoros: $Id: vflip.c,v 1.1 1991/05/10 15:49:07 khoros Exp $
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id: vflip.c,v 1.1 1991/05/10 15:49:07 khoros Exp $";
#endif

 /*
  * $Log: vflip.c,v $
 * Revision 1.1  1991/05/10  15:49:07  khoros
 * Initial revision
 *
  */ 

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1991, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as too the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1991 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: vflip.c
 >>>>
 >>>>      Program Name: vflip
 >>>>
 >>>>      Program Desc: 
 >>>>	
 >>>>	Flip (or reflect) an image
 >>>>	
 >>>>	
 >>>>            Author: Scott Wilson
 >>>>
 >>>> Date Last Updated: Wed Mar 27 23:49:06 1991
 >>>>
 >>>>          Routines: main- the main program for vflip
 >>>> 		 gw_usage - gives usage of the program
 >>>> 		 gw_args  - gets arguments of program from command line
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "vflip.h"


/****************************************************************
 *
 * Routine Name:  main program for vflip
 *
 *       Input:  
 *       -i  input image  
 *       -o  output image 
 *
 *
 *       [-t] flip top/bottom
 *       [-r] flip right/left
 *
 *
 *
 ****************************************************************/


main(argc, argv)
int argc;
char *argv[];
{

/* -main_variable_list */
        struct xvimage *image,*readimage();
/* -main_variable_list_end */

	program = VStrcpy(argv[0]);
	ac = argc;
	av = argv;

	khoros_init();

	gw_get_args();

/* -main_before_lib_call */
        if (check_args()) exit(1);

        image = readimage(vflip->i_file);
        if (image == NULL) exit(1);
        CHECKINPUT(program, image);
/* -main_before_lib_call_end */

/* -main_library_call */
        if (!lvflip(image,vflip->r_logic,vflip->t_logic))
          {
            (void)fprintf(stderr,"lvflip failed\n");
            exit(1);
          }
/* -main_library_call_end */

/* -main_after_lib_call */
        writeimage(vflip->o_file,image);
/* -main_after_lib_call_end */


	khoros_close();
}


/****************************************************************
*
* Routine Name:  gw_usage 
*
*
* Purpose:  prints out the usage for vflip 
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_usage()
{
	fprintf(stderr, "vflip :\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\tFlip (or reflect) an image\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");

/* -usage_additions */
/* -usage_additions_end */

	fprintf(stderr,"\t-i  input image   (infile)\n");
	fprintf(stderr,"\t-o  output image  (outfile)\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\n");
	fprintf(stderr,"\t[-t]  flip top/bottom (boolean) [false]\n");
	fprintf(stderr,"\t[-r]  flip right/left (boolean) [true]\n");
	fprintf(stderr, " \n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\t[-V] Gives the version for vflip\n");
	fprintf(stderr, "\t[-U] Gives the usage for vflip\n");
	fprintf(stderr, "\t[-P] Prompts for command line options\n");
	fprintf(stderr, "\t[-A [file1]] Creates the answer file called vflip.ans or file1 \n");
	fprintf(stderr, "\t[-a [file1]] Uses vflip.ans or file1 as the answer file \n\n\n");

}


/****************************************************************
*
* Routine Name:  gw_getargs 
*
*
* Purpose:  gets arguments off command line for vflip
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_get_args()
{

char	*i_pstr = 
	"\nEnter: (i) input image  \n        {infile}: "; 
char	*o_pstr = 
	"\nEnter: (o) output image \n        {outfile}: "; 
char	*t_pstr = 
	"\nEnter: (t) flip top/bottom \n        {boolean, (y/n)  [no] }: "; 
char	*r_pstr = 
	"\nEnter: (r) flip right/left \n        {boolean, (y/n)  [yes] }: "; 
char  answer_file[512], temp[512];



int	fid = -1,
	error, query, ok, count;
	int prompt_flag = FALSE;

	/*
	 * print version if necessary
	 */
	vgparml(fid, "-V", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    printf ("\n%s: %s: Version %d.%d\n\n",
	    "vflip", RELEASE_NAME, RELEASE_MAJOR, RELEASE_MINOR);
	    exit(1);
	}

	/*
	 * print usage if necessary
	 */
	vgparml(fid, "-U", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    gw_usage();
	    exit(1);
	}

	/*
	 * see if interactive prompting is desired
	 */
	vgparml(fid, "-P", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	    prompt_flag = TRUE;

	/*
	 * see if answer file is to be used
	 */
	vgparms(fid, "-a", "noprompt", answer_file, 0, 0, &error);
	if (error == 0)
	{
	     if (VStrlen(answer_file) == 0)
		sprintf(answer_file, "vflip.ans");
	     while (fid < 0)
	     {
		if ((fid = open(answer_file, O_RDONLY, 0666))== -1)
		{
		    fprintf(stderr, "vflip: can't open answer file \"%s\"",
			    answer_file); 
		    fprintf(stderr, "please re-enter: ");
		    (void) gets(answer_file);
		    if (VStrlen(answer_file) == 0)
			sprintf(answer_file, "vflip.ans");
		}

	     }

	}

	/*
	 * allocate the vflip structure
	 */
	vflip = (vflip_struct *) 
	       calloc((unsigned)1, (unsigned) sizeof (vflip_struct));

	/*
	 * get required arguments for vflip
	 */
	if (prompt_flag == TRUE)
	    fprintf(stderr,"\nRequired Arguments:\n\n");

	if (!(vflip->i_flag = vget_infile(fid, prompt_flag,
			"-i", i_pstr, &(vflip->i_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(vflip->i_flag))
		{
		    fprintf(stderr, "\t\ti is required, please re-enter: \n");
		    vflip->i_flag = vget_infile(fid, prompt_flag,
		    "-i", i_pstr, &(vflip->i_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-i' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	if (!(vflip->o_flag = vget_outfile(fid, prompt_flag,
			"-o", o_pstr, &(vflip->o_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(vflip->o_flag))
		{
		    fprintf(stderr, "\t\to is required, please re-enter: \n");
		    vflip->o_flag = vget_outfile(fid, prompt_flag,
		    "-o", o_pstr, &(vflip->o_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-o' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	/*
	 * get optional arguments for vflip
	 */
	if (prompt_flag == TRUE)
	   fprintf(stderr,"\nOptional Arguments:\n\n");

	vflip->t_flag = vget_logic(fid, prompt_flag,
	"-t", t_pstr, &(vflip->t_logic), 0);

	vflip->r_flag = vget_logic(fid, prompt_flag,
	"-r", r_pstr, &(vflip->r_logic), 1);


}



